// Copyright (c) 2021 Tencent. All rights reserved.
// Author: tatemin

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:tencent_calls_engine/tuicall_observer.dart';

import 'tuicall_engine_platform_interface.dart';
import 'package:tencent_calls_engine/tuicall_define.dart';

/// An implementation of [TUICallEnginePlatform] that uses method channels.
class MethodChannelTUICallEngine extends TUICallEnginePlatform {
  final List<TUICallObserver> _observerList = [];

  @visibleForTesting
  final methodChannel = const MethodChannel('tuicall_engine');

  MethodChannelTUICallEngine() {
    methodChannel.setMethodCallHandler((call) async {
      if (call.method == "onTUICallObserver") {
        String observerName = call.arguments['method'];
        var observerParams = call.arguments['params'];

        _onTUICallObserver(observerName, observerParams);
      } else {
        debugPrint("onTUICallObserver: MethodNotImplemented ${call.arguments}");
      }
    });
  }

  @override
  Future<void> addObserver(TUICallObserver observer) async {
    _observerList.add(observer);
    methodChannel.invokeMethod("addObserver", {});
  }

  @override
  Future<void> removeObserver(TUICallObserver observer) async {
    _observerList.remove(observer);
    if (_observerList.isEmpty) {
      methodChannel.invokeMethod("removeObserver", {});
    }
  }

  void _onTUICallObserver(String observerName, var params) {
    for (var observer in _observerList) {
      switch (observerName) {
        case "onError":
        {
          observer.onError!(params["code"], params['message']);
          break;
        }

        case "onCallReceived":
        {
          TUICallMediaType callMediaType =
              TUICallMediaType.values[params['callMediaType']];
          observer.onCallReceived!(
              params['callerId'],
              params['calleeIdList'] as List<String>,
              params['groupId'],
              callMediaType);
          break;
        }

        case "onCallCancelled":
        {
          observer.onCallCancelled!(params['callerId']);
          break;
        }

        case "onCallBegin":
        {
          final roomIdMap = Map<String, int>.from(params['roomId']);
          TUIRoomId roomId = TUIRoomId(intRoomId: roomIdMap['intRoomId']!);
          TUICallMediaType callMediaType =
              TUICallMediaType.values[params['callMediaType']];
          TUICallRole callRole = TUICallRole.values[params['callRole']];
          observer.onCallBegin!(roomId, callMediaType, callRole);
          break;
        }

        case "onCallEnd":
        {
          TUIRoomId roomId =
              TUIRoomId(intRoomId: params['roomId']['intRoomId']);
          TUICallMediaType callMediaType =
              TUICallMediaType.values[params['callMediaType']];
          TUICallRole callRole = TUICallRole.values[params['callRole']];
          double totalTime = params['totalTime'];
          observer.onCallEnd!(roomId, callMediaType, callRole, totalTime);
          break;
        }

        case "onCallMediaTypeChanged":
        {
          TUICallMediaType oldCallMediaType =
              TUICallMediaType.values[params['oldCallMediaType']];
          TUICallMediaType newCallMediaType =
              TUICallMediaType.values[params['newCallMediaType']];
          observer.onCallMediaTypeChanged!(
              oldCallMediaType, newCallMediaType);
          break;
        }

        case "onUserReject":
        {
          observer.onUserReject!(params['userId']);
          break;
        }

        case "onUserNoResponse":
        {
          observer.onUserNoResponse!(params['userId']);
          break;
        }

        case "onUserLineBusy":
        {
          observer.onUserLineBusy!(params['userId']);
          break;
        }

        case "onUserJoin":
        {
          observer.onUserJoin!(params['userId']);
          break;
        }

        case "onUserLeave":
        {
          observer.onUserLeave!(params['userId']);
          break;
        }

        case "onUserVideoAvailable":
        {
          observer.onUserVideoAvailable!(
              params['userId'], params['isVideoAvailable']);
          break;
        }

        case "onUserAudioAvailable":
        {
          observer.onUserAudioAvailable!(
              params['userId'], params['isAudioAvailable']);
          break;
        }

        case "onUserVoiceVolumeChanged":
        {
          final volumeMap = Map<String, int>.from(params['volumeMap']);
          observer.onUserVoiceVolumeChanged!(volumeMap);
          break;
        }

        case "onUserNetworkQualityChanged":
        {
          List returnNetworkQualityList = params['networkQualityList'];
          List<TUINetworkQualityInfo> networkQualityLists = [];
          for(final element in returnNetworkQualityList) {
            var userId = (element['userId'] == null) ? "" : element['userId'];
            TUINetworkQuality quality = TUINetworkQuality.values[element["networkQuality"]];
            networkQualityLists.add(TUINetworkQualityInfo(userId: userId, quality: quality));
          }
          observer.onUserNetworkQualityChanged!(networkQualityLists);
          break;
        }

        case "onKickedOffline":
        {
          observer.onKickedOffline!();
          break;
        }

        case "onUserSigExpired":
        {
          observer.onUserSigExpired!();
          break;
        }
      }
    }
  }
}
